#!/usr/bin/env ruby

$:.unshift('../../lib')

require 'webrick/https'
require 'getopts'

getopts "c", 'r:', 'p:8808'

dir = File::dirname(File::expand_path(__FILE__))

if $OPT_c
  verify_mode = OpenSSL::SSL::VERIFY_PEER
  verify_callback = lambda{|ok, ctx|
    p [ ok, ctx.error_string, ctx.current_cert.subject ]
    return true
  }
else
  verify_mode = OpenSSL::SSL::VERIFY_NONE
  verify_callback = nil
end

pkey = cert = cert_name = nil
begin
  data = open(File::join(dir, "conf/sample.key")){|io| io.read }
  pkey = OpenSSL::PKey::RSA.new(data)
  data = open(File::join(dir, "conf/sample.crt")){|io| io.read }
  cert = OpenSSL::X509::Certificate.new(data)
rescue
  $stderr.puts "Switching to use self-signed certificate"
  cert_name = [
    [ "CN", WEBrick::Utils::getservername ], [ "OU", "SampleServers" ],
    [ "O", "WEBrick" ], [ "C", "JP" ]
  ]
end

s=WEBrick::HTTPServer.new(
  :BindAddress       => "0.0.0.0",
  :Port              => $OPT_p.to_i, 
  :Logger            => WEBrick::Log::new($stderr, WEBrick::Log::DEBUG),
  :DocumentRoot      => $OPT_r || File::join(dir, "/htdocs"),
  :SSLEnable         => true,
  :SSLVerifyClient   => verify_mode,
  :SSLVerifyCallback => verify_callback,
  :SSLCertificate    => cert,
  :SSLPrivateKey     => pkey,
  :SSLCertName       => cert_name
)
trap("INT"){ s.shutdown }
s.start
