#!/usr/bin/env ruby
#
# leakproxy.rb -- TCP application monitoring tool
#
# Author: IPR -- Internet Programming with Ruby -- writers
# Copyright (c) 2003 Internet Programming with Ruby writers. All rights
# reserved.
#
# $IPR: leakproxy.rb,v 1.1 2003/02/20 11:04:22 gotoyuzo Exp $
require 'webrick/compat'
require 'webrick/server'

class LeakProxy < WEBrick::GenericServer
  def run(cli)
    svr = TCPSocket.new(@config[:RemoteAddress], @config[:RemotePort])
    begin
      leak(cli, svr)
    rescue
      svr.close
    end
  end

  def highlight(str, code)
    @config[:Highlight] ? "\e[#{code}m#{str}\e[0m" : str
  end

  def process_data(direction, data)
    data.each{|line|
      if direction == :CLIENT_TO_SERVER
        print "C: ", highlight(line.dump, 0), "\n" # no effect
      else
        print "S: ", highlight(line.dump, 1), "\n" # bold face
      end
    }
  end

  def leak(cli, svr)
    while(true)
      ary = IO::select([cli, svr])
      if ary[0].member?(cli)
        data = cli.sysread(4096)
        process_data(:CLIENT_TO_SERVER, data)
        svr.syswrite(data)
      end
      if ary[0].member?(svr)
        data = svr.sysread(4096)
        process_data(:SERVER_TO_CLIENT, data)
        cli.syswrite(data)
      end
    end
  end
end   # LeakProxy

if $0 == __FILE__
  local_port = ARGV.shift
  if hostport = ARGV.shift
    host, port = hostport.split(':', 2)
  end
  if local_port.nil? || host.nil? || port.nil? || ARGV.size > 0
    $stderr.puts("#{File::basename($0)} local_port host:port")
    exit 2
  end
  lp = LeakProxy.new(
    :Port          => local_port,
    :RemoteAddress => host,
    :RemotePort    => port,
    :Highlight     => STDOUT.tty?,
    :Logger => WEBrick::Log::new($stderr, WEBrick::Log::DEBUG)
  )
  trap(:INT){ lp.shutdown }
  lp.start
end
