#! /usr/local/bin/ruby -Ke

require 'ndtp'
require 'webrick'
require 'fileutils'

class WebDic < WEBrick::HTTPServlet::AbstractServlet
  def initialize(config, ndtp, gaiji_url)
    @config = config
    @ndtp   = ndtp
    @gaiji_url = gaiji_url
  end

  def do_GET(req, res)
    @word  = req.query['word']
    @sdict = req.query['sdict']
    @dict  = req.query['dict']
    @point = req.query['point']
    @cmd   = req.query['cmd']

    @ndtp.synchronize{
      begin
        @ndtp.version
      rescue
        @ndtp.start
      end
    }

    htword = WEBrick::HTMLUtils::escape(@word)
    body =<<-_end_of_html_
<HTML>
 <BODY>
  <FORM method="GET" action="#{req.script_name}/">
   <INPUT type="hidden" name="cmd" value="scan">
   word: <INPUT name="word" value="#{htword}">
   <INPUT type="submit" value="scan"><BR>
   dict: <SELECT name="sdict">
   <OPTION value='all'>-- all dictionaries --</OPTION>
    _end_of_html_

    body << @ndtp.dict_list.collect{|number, dict, name|
      hdict = WEBrick::HTMLUtils::escape(dict)
      hname = WEBrick::HTMLUtils::escape(name)
      if @sdict == dict
        "<OPTION value='#{hdict}' selected>#{hname}</OPTION>\n"
      else
        "<OPTION value='#{hdict}'>#{hname}</OPTION>\n"
      end
    }.join
    body << "</SELECT>\n"
    body << "</FORM><HR>\n"

    case @cmd
    when 'scan'
      body << scan(@sdict||"all", @word).to_s
    when 'consult'
      body << consult(@dict, @point).to_s
    end

    body << <<-_end_of_html_
    <HR>
    <ADDRESS>
     Copyright (c) 2002-2003 GOTOU Yuuzou
     <A href="mailto:gotoyuzo@notwork.org">&lt;gotoyuzo@notwork.org&gt;</A><BR>
     $Id: webdic.rb,v 1.11 2003/05/31 22:29:44 gotoyuzo Exp $
    </ADDRESS>
  </BODY>
</HTML>
    _end_of_html_

    res.body = body

    res['content-type'] = 'text/html'
  end

  def decorate_uri(uri)  
    tmp = uri.dup
    tmp << "&sdict=#{@sdict}" if @sdict
    tmp << "&word=#{WEBrick::HTMLUtils::escape(@word)}" if @word
    tmp
  end

  def consult(dict, point)
    return unless dict
    return unless point

    result = nil
    @ndtp.synchronize{
      @ndtp.select_dict(dict)
      if size = @ndtp.get_gaiji_size.first
        @ndtp.set_gaiji_size(size)
      end
      result = @ndtp.consult(point)
    }
    result = markup(result, dict)
    result.gsub!(/\n/, "<BR>\n")
    result.sub!(/\A/, "<P>\n")
    result.sub!(/\Z/, "\n</P>")
    result
  end

  def scan(dic, word)
    return unless word
    result = ''
    @ndtp.dict_list.each{|number, dict, name|
      next if dic != 'all' && dic != dict
      entries = @ndtp.synchronize{
        @ndtp.select_dict(number)
        if size = @ndtp.get_gaiji_size.first
          @ndtp.set_gaiji_size(size)
        end
        @ndtp.entry(word)
      }
      result << "<H2> " << WEBrick::HTMLUtils::escape(name) << "</H2>\n"
      result << "<UL>\n"
      entries.each{|index, point|
        href = "?cmd=consult&dict=#{dict}&point=#{point}"
        href = decorate_uri(href)
        href = WEBrick::HTMLUtils::escape(href)
        result << "<LI>"
        result << "<A href='#{href}'>" << markup(index, dict) << "</A>"
        result << "</LI>\n"
      }
      result << "</UL>\n"
    }
    result
  end

  def markup(text, dict)
    ret = ""
    while i = text.index(/#{NDTP::REFERENCE_REGEX}|#{NDTP::GAIJI_REGEX}/)
      len = $&.length
      point, gaiji_id = $1, $2
      ret << WEBrick::HTMLUtils::escape(text[0...i])
      if point
        href = "?cmd=consult&dict=#{dict}&point=#{point}"
        href = decorate_uri(href)
        href = WEBrick::HTMLUtils::escape(href)
        ret << "(<A href='#{href}'><I>see</I></A>)"
      elsif gaiji_id
        img = File::join(@gaiji_url, dict, gaiji_id+".png")
        img = WEBrick::HTMLUtils::escape(img)
        ret << "<IMG src='#{img}' alt='.'>"
      end
      text[0...(i+len)] = ""
    end
    ret << WEBrick::HTMLUtils::escape(text)
    ret
  end
end

def _setup_webdic(server, mount_point)
  gaiji_dir = File::join(DocumentRoot, mount_point, "gaiji")
  gaiji_dir = File::expand_path(gaiji_dir)
  $stderr.puts("_setup_webdic: gaiji_dir = #{gaiji_dir}")

  gaiji_url = mount_point == "/" ?
    "/gaiji" : gaiji_url = File::join(mount_point, "gaiji")

  server.mount(mount_point, WebDic, NDTP.new(:Debug=>$DEBUG), gaiji_url)
  server.mount(gaiji_url, WEBrick::HTTPServlet::FileHandler, gaiji_dir)
end

if __FILE__ == $0
  require 'getopts'
  getopts "", 'p:8808'

  s = WEBrick::HTTPServer.new(
    :Port         => $OPT_p.to_i,
    :StartThreads => 2
  )
  _setup_webdic(s, "/dict")
  trap(:INT){ s.shutdown }
  s.start
end
